import { Context } from '@nuxt/types';
import { RequestHandler, Request } from 'express';

interface GraphqlMiddlewarePluginConfig {
    enabled?: boolean;
    cacheInBrowser?: boolean;
    cacheInServer?: boolean;
}
declare class GraphqlMiddlewarePlugin {
    baseURL: string;
    headers: any;
    beforeRequestFn: Function | undefined;
    cache?: Map<string, any>;
    context: any;
    constructor(baseURL: string, headers: any, useCache: boolean, context: Context);
    getPluginHeaderValue(): {
        'Nuxt-Graphql-Middleware-Route': any;
    };
    beforeRequest(fn: Function): void;
    /**
     * Perform a GraphQL query via the middleware.
     */
    query(name: string, variables?: any, headers?: any): Promise<any>;
    /**
     * Perform a GraphQL mutation via the middleware.
     */
    mutate(name: string, variables?: any, headers?: any): Promise<any>;
}

interface GraphqlServerMiddlewareConfig {
    middleware?: RequestHandler;
    fetchOptions?: any;
    buildHeaders?: (req: Request, name: string, type: string) => any;
    buildEndpoint?: (req: Request) => string;
    onQueryResponse?: any;
    onQueryError?: any;
    onMutationResponse?: any;
    onMutationError?: any;
}

interface GraphqlMiddlewareCodegenConfig {
    enabled?: boolean;
    skipSchemaDownload?: boolean;
    resolvedQueriesPath: string;
    schemaOutputPath: string;
    typesOutputPath: string;
    schemaOptions: any;
}

interface GraphqlMiddlewareConfig {
    graphqlServer: string;
    typescript?: GraphqlMiddlewareCodegenConfig;
    endpointNamespace?: string;
    debug: boolean;
    queries: Record<string, string>;
    mutations: Record<string, string>;
    outputPath: string;
    plugin?: GraphqlMiddlewarePluginConfig;
    server?: GraphqlServerMiddlewareConfig;
}

declare module 'vue/types/vue' {
    interface Vue {
        readonly $graphql: GraphqlMiddlewarePlugin;
    }
}
declare module 'vuex/types/index' {
    interface Store<S> {
        readonly $graphql: GraphqlMiddlewarePlugin;
    }
}
declare module '@nuxt/types' {
    interface NuxtAppOptions {
        readonly $graphql: GraphqlMiddlewarePlugin;
    }
    interface Context {
        readonly $graphql: GraphqlMiddlewarePlugin;
    }
    interface Configuration {
        graphqlMiddleware?: GraphqlMiddlewareConfig;
    }
}
